package com.tocea.easycoverage.framework.junit;

import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import junit.framework.TestSuite;

import com.tocea.easycoverage.framework.CheckersRunner;
import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.MethodChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.utils.Utils;

public class JUnitTestSuiteProvider {

	private int						timeout	= 0;
	private final CheckersRunner	runner	= new CheckersRunner();

	public TestSuite getTestSuite() {
		return getTestSuite("Easycoverage Test Suite");
	}

	public TestSuite getTestSuite(final String name) {
		final TestSuite suite = new TestSuite(name);

		final ExecutorService executorService = Executors.newCachedThreadPool();

		for (final Class<?> clazz : getClasses()) {
			final TestSuite classSuite = new TestSuite(clazz.getName());
			for (final ClassChecker checker : runner.getCheckers(clazz))
				classSuite.addTest(new EasycoverageTestCase(checker,
						executorService, timeout));
			for (final Method method : Utils.getDeclaredPublicMethods(clazz)) {
				final TestSuite methodSuite = new TestSuite(Utils
						.getQualifiedName(method).replace('(', '[')
						.replace(')', ']'));
				for (final MethodChecker checker : runner.getCheckers(method))
					methodSuite.addTest(new EasycoverageTestCase(checker,
							executorService, timeout));
				if (methodSuite.countTestCases() > 0)
					classSuite.addTest(methodSuite);
			}
			if (classSuite.countTestCases() > 0)
				suite.addTest(classSuite);
		}

		return suite;
	}

	public IInstanceProvider getProvider() {
		return runner.getProvider();
	}

	public void setProvider(final IInstanceProvider provider) {
		runner.setProvider(provider);
	}

	public Set<Class<?>> getClasses() {
		return runner.getClasses();
	}

	public void addClass(final Class<?> clazz) {
		runner.addClass(clazz);
	}

	public void addClasses(final Collection<Class<?>> classes) {
		runner.addClasses(classes);
	}

	/**
	 * @deprecated Since 0.2.3. Use {@link #addClasses(Collection)} instead.
	 */
	@Deprecated
	public void setClasses(final Set<Class<?>> classes) {
		runner.addClasses(classes);
	}

	public int getTimeout() {
		return timeout;
	}

	public void setTimeout(final int timeout) {
		this.timeout = timeout;
	}

	public Set<Class<? extends ClassChecker>> getClassCheckers() {
		return runner.getClassCheckers();
	}

	public void addClassChecker(final Class<? extends ClassChecker> checker) {
		runner.addClassChecker(checker);
	}

	public void addClassCheckers(
			final Collection<Class<? extends ClassChecker>> checkers) {
		runner.addClassCheckers(checkers);
	}

	/**
	 * @deprecated Since 0.2.3. Use {@link #addClassCheckers(Collection)}
	 *             instead.
	 */
	@Deprecated
	public void setClassCheckers(
			final Set<Class<? extends ClassChecker>> classCheckers) {
		addClassCheckers(classCheckers);
	}

	public Set<Class<? extends MethodChecker>> getMethodCheckers() {
		return runner.getMethodCheckers();
	}

	public void addMethodChecker(final Class<? extends MethodChecker> checker) {
		runner.addMethodChecker(checker);
	}

	public void addMethodCheckers(
			final Collection<Class<? extends MethodChecker>> checkers) {
		runner.addMethodCheckers(checkers);
	}

	/**
	 * @deprecated Since 0.2.3. Use {@link #addMethodCheckers(Collection)}
	 *             instead.
	 */
	@Deprecated
	public void setMethodCheckers(
			final Set<Class<? extends MethodChecker>> methodCheckers) {
		runner.addMethodCheckers(methodCheckers);
	}

}
